Date: 2025-06-20
Package: CHNOSZ
Version: 2.2.0
Title: Thermodynamic Calculations and Diagrams for Geochemistry
Authors@R: c(
    person("Jeffrey", "Dick", , "j3ffdick@gmail.com", role = c("aut", "cre"),
      comment = c(ORCID = "0000-0002-0687-5890"))
    )
Author: Jeffrey Dick [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0687-5890>)
Maintainer: Jeffrey Dick <j3ffdick@gmail.com>
Depends: R (>= 3.1.0)
Suggests: tinytest, knitr, rmarkdown, tufte, canprot (>= 2.0.0)
Imports: grDevices, graphics, stats, utils
Description: An integrated set of tools for thermodynamic calculations in
  aqueous geochemistry and geobiochemistry. Functions are provided for writing
  balanced reactions to form species from user-selected basis species and for
  calculating the standard molal properties of species and reactions, including
  the standard Gibbs energy and equilibrium constant. Calculations of the
  non-equilibrium chemical affinity and equilibrium chemical activity of species
  can be portrayed on diagrams as a function of temperature, pressure, or
  activity of basis species; in two dimensions, this gives a maximum affinity or
  predominance diagram. The diagrams have formatted chemical formulas and axis
  labels, and water stability limits can be added to Eh-pH, oxygen fugacity-
  temperature, and other diagrams with a redox variable. The package has been
  developed to handle common calculations in aqueous geochemistry, such as
  solubility due to complexation of metal ions, mineral buffers of redox or pH,
  and changing the basis species across a diagram ("mosaic diagrams"). CHNOSZ
  also implements a group additivity algorithm for the standard thermodynamic
  properties of proteins.
Encoding: UTF-8
License: GPL-3
VignetteBuilder: knitr
URL: https://chnosz.net/,
        https://r-forge.r-project.org/projects/chnosz/
NeedsCompilation: yes
Packaged: 2025-06-19 23:40:45 UTC; jedick
Repository: CRAN
Date/Publication: 2025-06-20 04:50:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:28:04 UTC; unix
Archs: CHNOSZ.so.dSYM
