# exports starting with functions used in vignettes, examples, demos, tests 20170224
export(
# anintro vignette
  "info", "makeup", "as.chemical.formula", "ZC",
  "subcrt", "T.units", "P.units", "E.units", "convert",
  "axis.label", "expr.species", "basis", "describe.reaction",
  "species", "affinity", "swap.basis", "diagram", "mosaic",
  "water.lines", "mod.buffer", "thermo.plot.new",
  "describe.property", "describe.basis", "equilibrate",
  "aminoacids", "ZC.col",
  "pinfo", "protein.length", "protein.formula",
  "protein.basis", "add.protein",
  "unitize",
  "thermo.refs", "mod.OBIGT",
# examples
  "examples", "demos", "mtitle",
  "list2array", "slice", "dimSums", "slice.affinity",
  "add.OBIGT", "RH2OBIGT",
  "expr.property", "expr.units",
  "mass", "entropy", "GHS", "water",
  "i2A",
  "dPdTtr", "Ttr",
  "rho.IAPWS95", "IAPWS95", "water.AW90", "WP02.auxiliary", "water.IAPWS95",
  "getrank", "parent", "sciname", "allparents", "getnodes", "getnames",
  "protein.OBIGT", "which.pmax",
  "equil.boltzmann", "equil.reaction", "find.tp",
  "ionize.aa", "MP90.cp",
  "ratlab",
  "EOSregress", "EOScoeffs", "EOSplot", "EOSvar",
# demos
  "palply",
  "label.plot",
  "basis.logact",
  "label.figure", "syslab",
# equilibrium vignette
  "usrfig",
# ecipex package
  "count.elements",
# (no other functions are used in the tests)
# other exported functions that are not used above
  "check.EOS", "check.GHS", "check.OBIGT",
  "V_s_var", "Cp_s_var",
  "EOSlab", "EOScalc",
  "basis.elements", "element.mu", "ibasis",
  "water.SUPCRT92",
  "nonideal",
# added 20170301 or later
  "GHS_Tr", "calculateDensity", "calculateGibbsOfWater",
  "calculateEpsilon", "calculateQ", "water.DEW", "Berman",
  "bgamma",
# added 20171121 or later
  "dumpdata", "thermo.axis", "solubility", "NaCl",
# added 20190213 or later
  "CHNOSZ", "thermo", "reset", "OBIGT", "retrieve", "moles",
  "lNaCl", "lS", "lT", "lP", "lTP", "lex",
# added 20200716 or later
  "mash", "mix", "rebalance",
# added 20220324
  "logK.to.OBIGT",
# added 20220416
  "rank.affinity",
# added 20220620
  "stack_mosaic"
)

# Load shared objects
# Refer to all C/Fortran routines by their name prefixed by C_
useDynLib(CHNOSZ, .registration = TRUE, .fixes = "C_")

# Imports from default packages
importFrom("grDevices", "dev.cur", "dev.off", "extendrange",
  "heat.colors", "png", "rainbow", "topo.colors", "dev.list",
  "contourLines", "col2rgb", "rgb", "adjustcolor")
importFrom("graphics", "abline", "axTicks", "axis", "barplot", "box",
  "contour", "image", "legend", "lines", "mtext", "par", "plot",
  "plot.new", "plot.window", "points", "rect", "text", "title")
importFrom("stats", "D", "as.formula", "cor", "lm", "loess.smooth",
  "na.omit", "predict.lm", "qqline", "qqnorm", "sd", "splinefun",
  "uniroot", "median", "predict", "smooth.spline", "optimize", "formula")
importFrom("utils", "browseURL", "capture.output", "combn", "demo",
  "example", "head", "installed.packages", "read.csv", "tail",
  "write.csv", "write.table", "read.table", "packageDescription")
# 20190420 new R functions for HCL color palettes
if (getRversion() >= "3.6.0") importFrom("grDevices", "hcl.pals", "hcl.colors")
