<!-- badges: start -->
[![CRAN](https://img.shields.io/badge/dynamic/yaml?url=https%3A%2F%2Fcloud.r-project.org%2Fweb%2Fpackages%2FCHNOSZ%2FDESCRIPTION&query=%24.Version&logo=r&label=CRAN&color=4bc51e)](https://cran.r-project.org/package=CHNOSZ)
[![R-CMD-check](https://github.com/jedick/CHNOSZ/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/jedick/CHNOSZ/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

This is a mirror of the [primary repository on R-Forge](https://r-forge.r-project.org/projects/chnosz/).

Please see the [CHNOSZ website](https://chnosz.net) for more information.

**NEW**: For questions about CHNOSZ, please use the [Discussions forum](https://github.com/jedick/CHNOSZ/discussions) on GitHub.
